@@ -45,14 +45,18 @@
 #include <camel/camel-session.h>
 #include <camel/camel-stream-filter.h>
 #include <camel/camel-stream-mem.h>
+#include <camel/camel-folder-summary.h>
+#include <camel/camel-string-utils.h>
 
 static CamelOfflineFolderClass *parent_class = NULL;
 
+static CamelProperty exchange_property_list[] = {
+	{ CAMEL_EXCHANGE_FOLDER_CHECK_FOLDER, "check_folder", N_("Always check for new mail in this folder") },
+};
+
 /* Returns the class for a CamelFolder */
 #define CF_CLASS(so) CAMEL_FOLDER_CLASS (CAMEL_OBJECT_GET_CLASS(so))
 
-static void folder_sync (CamelFolder *folder, gboolean expunge,
-			 CamelException *ex);
 static void exchange_expunge (CamelFolder *folder, CamelException *ex);
 static void append_message (CamelFolder *folder, CamelMimeMessage *message,
 			    const CamelMessageInfo *info, char **appended_uid,
@@ -84,13 +88,15 @@ static void   transfer_messages_the_hard_way (CamelFolder *source,
 static void refresh_info (CamelFolder *folder, CamelException *ex);
 static void exchange_sync (CamelFolder *folder, gboolean expunge, CamelException *ex);
 
+static int exchange_getv (CamelObject *object, CamelException *ex, CamelArgGetV *args);
+static int exchange_setv (CamelObject *object, CamelException *ex, CamelArgV *args);
+
 static void
 class_init (CamelFolderClass *camel_folder_class)
 {
 	parent_class = CAMEL_OFFLINE_FOLDER_CLASS (camel_type_get_global_classfuncs (camel_offline_folder_get_type ())) ;
 
 	/* virtual method definition */
-	camel_folder_class->sync = folder_sync;
 	camel_folder_class->expunge = exchange_expunge;
 	camel_folder_class->append_message = append_message;
 	camel_folder_class->get_message = get_message;
@@ -100,6 +106,9 @@ class_init (CamelFolderClass *camel_folder_class)
 	camel_folder_class->transfer_messages_to = transfer_messages_to;
 	camel_folder_class->refresh_info = refresh_info;
 	camel_folder_class->sync = exchange_sync;
+
+ 	((CamelObjectClass *)camel_folder_class)->getv = exchange_getv;
+	((CamelObjectClass *)camel_folder_class)->setv = exchange_setv;
 }
 
 #define CAMEL_EXCHANGE_SERVER_FLAGS \
@@ -141,6 +150,8 @@ camel_exchange_folder_get_type (void)
 	static CamelType camel_exchange_folder_type = CAMEL_INVALID_TYPE;
 
 	if (camel_exchange_folder_type == CAMEL_INVALID_TYPE) {
+		int i;
+
 		camel_exchange_folder_type = camel_type_register (
 			camel_offline_folder_get_type (),
 			"CamelExchangeFolder",
@@ -150,50 +161,15 @@ camel_exchange_folder_get_type (void)
 			NULL,
 			(CamelObjectInitFunc) init,
 			(CamelObjectFinalizeFunc) finalize );
+
+		/* only localize here, do not create GSList, we do not want to leak */
+		for (i = 0; i < sizeof (exchange_property_list)/sizeof (exchange_property_list [0]); i++)
+			exchange_property_list [i].description = _(exchange_property_list [i].description);
 	}
 
 	return camel_exchange_folder_type;
 }
 
-
-static void
-folder_sync (CamelFolder *folder, gboolean expunge, CamelException *ex)
-{
-	CamelExchangeFolder *exch = CAMEL_EXCHANGE_FOLDER (folder);
-	GPtrArray *summary, *uids;
-	CamelMessageInfo *info;
-	CamelFolder *trash;
-	int i;
-
-	/* Give the backend a chance to process queued changes. */
-	camel_stub_send (exch->stub, NULL, CAMEL_STUB_CMD_SYNC_FOLDER,
-			 CAMEL_STUB_ARG_FOLDER, folder->full_name,
-			 CAMEL_STUB_ARG_END);
-
-	/* If there are still deleted messages left, we need to delete
-	 * them the hard way.
-	 */
-	summary = camel_folder_get_summary (folder);
-	uids = g_ptr_array_new ();
-	for (i = 0; i < summary->len; i++) {
-		info = summary->pdata[i];
-		if (!(camel_message_info_flags(info) & CAMEL_MESSAGE_DELETED))
-			continue;
-		g_ptr_array_add (uids, (char *)camel_message_info_uid (info));
-	}
-	if (uids->len) {
-		trash = camel_store_get_trash (folder->parent_store, ex);
-		if (trash) {
-			transfer_messages_the_hard_way (folder, uids, trash,
-							NULL, TRUE, ex);
-		}
-	}
-	g_ptr_array_free (uids, TRUE);
-	camel_folder_free_summary (folder, summary);
-
-	camel_folder_summary_save (folder->summary);
-}
-
 static void
 refresh_info (CamelFolder *folder, CamelException *ex)
 {
@@ -254,7 +230,7 @@ append_message_data (CamelFolder *folder, GByteArray *message,
 
 	if (camel_stub_send (exch->stub, ex, CAMEL_STUB_CMD_APPEND_MESSAGE,
 			     CAMEL_STUB_ARG_FOLDER, folder->full_name,
-			     CAMEL_STUB_ARG_UINT32, camel_message_info_flags(info),
+			     CAMEL_STUB_ARG_UINT32, info? camel_message_info_flags(info): 0,
 			     CAMEL_STUB_ARG_STRING, subject,
 			     CAMEL_STUB_ARG_BYTEARRAY, message,
 			     CAMEL_STUB_ARG_RETURN,
@@ -489,15 +465,12 @@ search_by_expression (CamelFolder *folder, const char *expression,
 		      CamelException *ex)
 {
 	CamelFolderSearch *search;
-	GPtrArray *matches, *summary, *response;
+	GPtrArray *matches, *response;
 	int i;
 
 	search = camel_exchange_search_new ();
 	camel_folder_search_set_folder (search, folder);
-	summary = camel_folder_get_summary (folder);
-	camel_folder_search_set_summary (search, summary);
-	matches = camel_folder_search_execute_expression (search, expression, ex);
-	camel_folder_free_summary (folder, summary);
+	matches = camel_folder_search_search (search, expression, NULL, ex);
 
 	if (matches) {
 		response = g_ptr_array_new ();
@@ -517,41 +490,16 @@ search_by_uids (CamelFolder *folder, const char *expression,
 		GPtrArray *uids, CamelException *ex)
 {
 	CamelFolderSearch *search;
-	GPtrArray *matches, *summary, *response;
-	int i;
-
-	summary = g_ptr_array_new();
-	for (i = 0; i < uids->len; i++) {
-		CamelMessageInfo *info;
-
-		info = camel_folder_get_message_info (folder, uids->pdata[i]);
-		if (info)
-			g_ptr_array_add (summary, info);
-	}
-
-	if (summary->len == 0)
-		return summary;
+	GPtrArray *matches;
 
 	search = camel_exchange_search_new ();
 	camel_folder_search_set_folder (search, folder);
-	camel_folder_search_set_summary (search, summary);
+	camel_folder_search_set_summary (search, uids);
 	matches = camel_folder_search_execute_expression (search, expression, ex);
 
-	if (matches) {
-		response = g_ptr_array_new ();
-		for (i = 0; i < matches->len; i++)
-			g_ptr_array_add (response, g_strdup (matches->pdata[i]));
-		camel_folder_search_free_result (search, matches);
-	} else
-		response = NULL;
-
-	for (i = 0; i < summary->len; i++)
-		camel_folder_free_message_info (folder, summary->pdata[i]);
-	g_ptr_array_free (summary, TRUE);
-
 	camel_object_unref (CAMEL_OBJECT (search));
 
-	return response;
+	return matches;
 }
 
 static void
@@ -812,7 +760,7 @@ camel_exchange_folder_add_message (CamelExchangeFolder *exch,
 	}
 	camel_object_unref (CAMEL_OBJECT (msg));
 
-	info->uid = g_strdup (uid);
+	info->uid = camel_pstring_strdup (uid);
 	einfo->info.flags = flags;
 	einfo->info.size = size;
 	einfo->href = g_strdup (href);
@@ -1104,9 +1052,12 @@ camel_exchange_folder_construct (CamelFolder *folder, CamelStore *parent,
 		hrefs = g_ptr_array_new ();
 		g_ptr_array_set_size (hrefs, summary->len);
 
+		if (summary->len - camel_folder_summary_cache_size (folder->summary) > 50)
+			camel_folder_summary_reload_from_db (folder->summary, ex);
+		
 		for (i = 0; i < summary->len; i++) {
-			info = summary->pdata[i];
-			uids->pdata[i] = (char *)camel_message_info_uid (info);
+			uids->pdata[i] = g_strdup(summary->pdata[i]);
+			info = camel_folder_summary_uid (folder->summary, uids->pdata[i]);
 			flags->data[i] = ((CamelMessageInfoBase *)info)->flags & CAMEL_EXCHANGE_SERVER_FLAGS;
 			hrefs->pdata[i] = ((CamelExchangeMessageInfo *)info)->href;
 			//camel_tag_list_free (&((CamelMessageInfoBase *)info)->user_tags);
@@ -1152,7 +1103,8 @@ camel_exchange_folder_construct (CamelFolder *folder, CamelStore *parent,
 		camel_operation_end (NULL);
 		if (!ok)
 			return FALSE;
-		camel_folder_summary_save (folder->summary);
+		
+		camel_folder_summary_save_to_db (folder->summary, ex);
 	}
 
 	if (camel_exchange_summary_get_readonly (folder->summary))
@@ -1167,5 +1119,80 @@ exchange_sync (CamelFolder *folder, gboolean expunge, CamelException *ex)
 	if (expunge)
 		exchange_expunge (folder, ex);
 
-	camel_folder_summary_save (folder->summary);
+	camel_folder_summary_save_to_db (folder->summary, ex);
 }
+
+static int
+exchange_getv (CamelObject *object, CamelException *ex, CamelArgGetV *args)
+{
+	int i, count=0;
+	guint32 tag;
+
+	for (i = 0; i < args->argc; i++) {
+		CamelArgGet *arg = &args->argv[i];
+
+		tag = arg->tag;
+
+		switch (tag & CAMEL_ARG_TAG) {
+		case CAMEL_OBJECT_ARG_PERSISTENT_PROPERTIES:
+		case CAMEL_FOLDER_ARG_PROPERTIES: {
+			CamelArgGetV props;
+			int i;
+
+			props.argc = 1;
+			props.argv[0] = *arg;
+			((CamelObjectClass *)parent_class)->getv(object, ex, &props);
+
+			for (i = 0; i < sizeof (exchange_property_list)/sizeof (exchange_property_list [0]); i++)
+				*arg->ca_ptr = g_slist_append (*arg->ca_ptr, &exchange_property_list[i]);
+			break; }
+			/* exchange args */
+		case CAMEL_EXCHANGE_FOLDER_ARG_CHECK_FOLDER:
+			*arg->ca_int = ((CamelExchangeFolder *)object)->check_folder;
+			break;
+		default:
+			count++;
+			continue;
+		}
+
+		arg->tag = (tag & CAMEL_ARG_TYPE) | CAMEL_ARG_IGNORE;
+	}
+
+	if (count)
+		return ((CamelObjectClass *)parent_class)->getv (object, ex, args);
+
+	return 0;
+}
+
+static int
+exchange_setv (CamelObject *object, CamelException *ex, CamelArgV *args)
+{
+	int save = 0;
+	int i;
+	guint32 tag;
+
+	for (i = 0; i < args->argc; i++) {
+		CamelArg *arg = &args->argv[i];
+
+		tag = arg->tag;
+
+		switch (tag & CAMEL_ARG_TAG) {
+		case CAMEL_EXCHANGE_FOLDER_ARG_CHECK_FOLDER:
+			if (((CamelExchangeFolder *)object)->check_folder != arg->ca_int) {
+				((CamelExchangeFolder *)object)->check_folder = arg->ca_int;
+				save = 1;
+			}
+			break;
+		default:
+			continue;
+		}
+
+		arg->tag = (tag & CAMEL_ARG_TYPE) | CAMEL_ARG_IGNORE;
+	}
+
+	if (save)
+		camel_object_state_write (object);
+
+	return ((CamelObjectClass *)parent_class)->setv (object, ex, args);
+}
+
